//
// Scilab ( http://www.scilab.org/ ) - This file is part of Scilab
// Copyright (C) 2021 - Stephane MOTTELET
//
// This file is hereby licensed under the terms of the GNU GPL v3.0,
// For more information, see the COPYING file which you should have received
//
//--------------------------------------------------------------------------

#ifndef _IDAMANAGER_HXX_
#define _IDAMANAGER_HXX_

#include "OdeManager.hxx"

#include <ida/ida_impl.h>
#include <ida/ida.h>            /* prototypes for IDA fcts. and consts. */
#include <ida/ida_dense.h>     /* prototype for IDADense */
#include <ida/ida_direct.h>    /* prototypes for various DlsMat operations */

class IDAManager final : public OdeManager
{
    public :
    
    IDAManager(const std::wstring& callerName, const std::wstring& solverName, IDAManager* prevManager = NULL) : OdeManager(callerName, solverName)
    {
        m_prevManager = prevManager;
        m_isDAE = true;
        m_odeIsExtension = (prevManager != NULL);
    }

    ~IDAManager() {
        if (m_ida_mem != NULL) 
        {
            if (m_ida_mem != NULL) IDAFree(&m_ida_mem);
            m_ida_mem = NULL;
        }
    }

    OdeManager *getPreviousManager()
    {
        return m_prevManager;
    }
    
    void *create();
    bool initialize(char *errorMsg);
    bool computeIC(char *errorMsg);
    void errHandler(int error_code, const char *module, const char *function, char *msg);
    bool setUserData();
    bool setInitStep();
    bool setMinStep();
    bool setMaxStep();    
    bool setStopTime();
    bool setTolerances();
    bool setMaxOrder();
    bool setEventFunction();
    bool setSolverAndJacobian(char *errorMsg);
    bool setHandlerFunction();
    bool getRootInfo(int *piRootsFound);
    bool getCurrentTime(double *pdblCurrTime);
    bool getCurrentStep(double *pdblCurrStep);
    bool getLastOrder(int *piLastOrder);
    bool getDky(double dblRefTime, int iOrder, N_Vector Y);
    solverReturnCode doStep(double dblFinalTime, N_Vector Y, N_Vector Yp, double *pdblTime, solverTaskCode iKind);
    void saveContinuousExtensionVectors();
    void getInterpVectors(double *pdblNS, int iOrderPlusOne, double dblt0, double dblTUser, double dblStep, double *pdblVect, double *pdblVectd);

    types::Struct *getStats();

    std::vector<std::wstring> getAvailableMethods()
    {
        std::vector<std::wstring> availableMethods = {L"bdf"};
        return availableMethods;
    }

    int getMaxMethodOrder(std::wstring wstrMethod)
    {
        return 5;
    }

    private :
    
    IDAManager* m_prevManager;
    
    void *m_ida_mem = NULL;
    
    long int m_incStat[9] = {0,0,0,0,0,0,0,0,0};
};

#endif
