//
// Scilab ( http://www.scilab.org/ ) - This file is part of Scilab
// Copyright (C) 2021 - Stephane MOTTELET
//
// This file is hereby licensed under the terms of the GNU GPL v3.0,
// For more information, see the COPYING file which you should have received
//
//--------------------------------------------------------------------------

#include "OdeManager.hxx"

types::Double *OdeManager::createYOut(int iNbOut, int iSizeTSpan, bool bFlat)
{
    types::Double *pDblYOut;
    int iDimsY0 = m_pDblY0->getDims();

    if (bFlat == false && (iDimsY0 > 2 || (m_pDblY0->getCols() > 1 & iDimsY0 == 2)))
    {
        int *piDimsArrayY0 = m_pDblY0->getDimsArray();
        int *piDimsArrayYOut = new int(iDimsY0+1);
        for (int i=0; i<iDimsY0; i++)
        {
            piDimsArrayYOut[i] = piDimsArrayY0[i];
        }
        piDimsArrayYOut[iDimsY0] = iSizeTSpan;
        pDblYOut = new types::Double(iDimsY0+1, piDimsArrayYOut, m_odeIsComplex);
    }
    else
    {
        pDblYOut = new types::Double(iNbOut, iSizeTSpan, m_odeIsComplex);
    }
    return pDblYOut;
}

void OdeManager::createSolutionOutput(types::typed_list &out)
{
    // return a MList of "_odeSolution" type with method solution at internal steps
    // and a pointer to the OdeManager object. Extraction on this MList calls %_odeSolution_e and allows
    // to compute solution at arbitrary value of time by using method dense interpolant.
    bool bEvents = m_iNbEvents > 0;
    types::MList *pObj = new types::MList();
    types::String *pStr = new types::String(1,7+((int)m_isDAE) + ((int)bEvents)*(3+((int)m_isDAE)));
    int k=0;
    
    pStr->set(k++,L"_odeSolution");
    pStr->set(k++,L"solver");
    pStr->set(k++,L"method");
    pStr->set(k++,L"t");
    pStr->set(k++,L"y");
    if (m_isDAE)
    {
        pStr->set(k++,L"yp");        
    }
    if (bEvents == true)
    {
        pStr->set(k++,L"te");
        pStr->set(k++,L"ye");            
        if (m_isDAE)
        {
            pStr->set(k++,L"ype");
        }
        pStr->set(k++,L"ie");            
    }
    pStr->set(k++,L"stats");
    pStr->set(k++,L"idata");

    k = 0;
    pObj->set(k++,pStr);
    pObj->set(k++,new types::String(m_wstrSolver.c_str()));
    pObj->set(k++,new types::String(m_wstrMethod.c_str()));
    pObj->set(k++,getTOut());
    pObj->set(k++,getYOut());
    if (m_isDAE)
    {
        pObj->set(k++,getYpOut());
    }
    if (bEvents == true)
    {
        pObj->set(k++, getTEvent());
        pObj->set(k++, getYEvent());            
        if (m_isDAE)
        {
            pObj->set(k++,getYpEvent());
        }
        pObj->set(k++, getIndexEvent());            
    }
    pObj->set(k++, getStats());
    pObj->set(k++,new types::Pointer((void *)this));

    out.push_back(pObj); 
}

types::Double *OdeManager::getArrayFromVectors(std::vector<std::vector<double>> &m_vecY)
{
    types::Double *pDblY = createYOut(m_iNbEq, m_vecY.size());
    for (int i = 0; i < m_vecY.size(); i++)
    {
        double *pdbl = m_vecY[i].data();
        std::copy(pdbl, pdbl+m_iNbEq, pDblY->get() + i*m_iNbEq);
        if (m_odeIsComplex)
        {
            std::copy(pdbl+m_iNbEq, pdbl+2*m_iNbEq, pDblY->getImg() + i*m_iNbEq);                
        }
    }
    return pDblY;
}